@@ -4,7 +4,7 @@
     This file is part of Corrade.
 
     Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-                2017, 2018 Vladimír Vondruš <mosra@centrum.cz>
+                2017, 2018, 2019, 2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -29,10 +29,22 @@
  * @brief Forward declarations for the @ref Corrade::TestSuite namespace
  */
 
+#include <cstdint>
+
+#include "Corrade/Containers/Containers.h"
+
 namespace Corrade { namespace TestSuite {
 
+#ifndef DOXYGEN_GENERATING_OUTPUT
+enum class ComparisonStatusFlag: std::uint8_t;
+typedef Containers::EnumSet<ComparisonStatusFlag> ComparisonStatusFlags;
 template<class> class Comparator;
 /* Tester is always used through inheritance */
+#endif
+
+namespace Implementation {
+    template<class, class, class> struct ComparatorTraits;
+}
 
 }}
 
